/*
 *  ngtk/c/ngtk_radio.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 */

/*----------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>

ngtk_radio_item *ngtk_mk_radio_items(int n_items, char *list[n_items],
									 int rets[n_items])
{
	int i;
	int fill_size = sizeof(ngtk_radio_item);
	ngtk_radio_item *fill = (ngtk_radio_item *)malloc(fill_size);

    for (i=0;i<=n_items;i++) {
        fill_size += strlen(list[i])+sizeof(int);
        fill = (ngtk_radio_item *)realloc(fill, fill_size);
        fill[i].text = (char *)malloc(strlen(list[i])+1);
        strcpy(fill[i].text, list[i]);
        fill[i].ret = rets[i];
    }

    return(fill);
}

ngtk_radio_t *ngtk_mk_radio(char *title, int n_items, char *list[n_items-1],
							int rets[n_items-1])
{
    int i;
    int buf = 0;
    int max_len = 0;

    ngtk_radio_t *ret_radio = (ngtk_radio_t *)malloc(sizeof(ngtk_radio_t));

	ret_radio->title = (char *)malloc(strlen(title));
	strcpy(ret_radio->title, title);

	ret_radio->items = ngtk_mk_radio_items(n_items, list, rets);
	ret_radio->item_count = n_items;

    for (i=0;i<n_items;i++) {
        buf = strlen(list[i]);
        if (buf>max_len)
            max_len = buf;
    }
	ret_radio->max_item_len = max_len;
	ret_radio->boxed = 0;
	ret_radio->current_item = 0;
	ret_radio->visible = 0;

	return(ret_radio);
}

ngtk_radio_t *ngtk_show_radio(ngtk_radio_t *c_radio, int height, int width,
							  int top, int left, int cpair, int attr, int box,
							  int def_item)
{
	int offs = 0;
	int i;

	if (c_radio->visible == 1)
		return(c_radio);

	c_radio->current_item = def_item;
	c_radio->visible = 1;

	if (width <= c_radio->max_item_len)
		width = c_radio->max_item_len + 2;

    if (box) {
		if (width <= strlen(c_radio->title))
			width = strlen(c_radio->title) + 2;

		offs = 1;
		c_radio->boxed = 1;
		c_radio->box = ngtk_mkbox(height+2, width+2, top, left, cpair, attr,
								  c_radio->title, 0, 1, 1);
	}

	c_radio->container = ngtk_push_win(height, width, top+offs, left+offs,
									   cpair, attr, 0, 0);

	ngtk_clrwin(c_radio->container->win, 32, 1);

    for (i=0;i<=c_radio->item_count-1;i++)
        mvwprintw(c_radio->container->win, i, 1, "< > %s", c_radio->items[i].text);
	mvwaddch(c_radio->container->win, c_radio->current_item, 2, ACS_DIAMOND);
    wrefresh(c_radio->container->win);

	return(c_radio);
}


ngtk_radio_t *ngtk_hide_radio(ngtk_radio_t *c_radio)
{
	if (!c_radio->visible)
		return(c_radio);

	ngtk_pull_win(c_radio->container);
	ngtk_pop_win(c_radio->container);

	if (c_radio->boxed)
		ngtk_pop_win(c_radio->box);
	c_radio->visible = 0;

	return(c_radio);
}

ngtk_radio_t *ngtk_manage_radio(ngtk_radio_t *c_radio)
{
	WINDOW *win = c_radio->container->win;
	int buf = 0;

    while (buf != CR) {
		buf = ngtk_get_keysym();

		switch(buf) {
       	    case NGTK_KEY_UP:
           	    c_radio->prev_item = c_radio->current_item;
				if (c_radio->current_item <= 0)
					c_radio->current_item = c_radio->item_count-1;
				else
					c_radio->current_item--;
   	            break;

            case NGTK_KEY_DOWN:
   	            c_radio->prev_item = c_radio->current_item;
				if (c_radio->current_item >= c_radio->item_count-1)
					c_radio->current_item = 0;
				else
    	            c_radio->current_item++;
                break;

			case ESC:
				c_radio->current_item = -1;
				return(c_radio);
				break;

   	        default:
       	        break;
        }
        mvwaddch(win, c_radio->current_item, 2, ACS_DIAMOND);
		mvwaddch(win, c_radio->prev_item, 2, ' ');
        wrefresh(win);
    }
	return(c_radio);
}


void ngtk_destroy_radio(ngtk_radio_t *c_radio)
{
	int i;

	if (c_radio->visible)
		ngtk_hide_radio(c_radio);

	for (i=0;i<=c_radio->item_count;i++)
		free(c_radio->items[i].text);

	free(c_radio->title);
	free(c_radio);
}

ngtk_radio_t *ngtk_auto_pos_show_radio(ngtk_radio_t *c_radio, int height,
									   int width, int cpair, int attr,
									   int box, int def_item)
{
    int top;
    int left;

    if (height <= c_radio->item_count)
		height = c_radio->item_count;
	top = (LINES/2) - (height/2);
	left = (COLS/2) - (width/2);

	return(ngtk_show_radio(c_radio, height, width, top, left, cpair, attr,
						   box, def_item));
}

ngtk_radio_t *ngtk_auto_size_show_radio(ngtk_radio_t *c_radio, int top,
										int left, int cpair, int attr,
										int box, int def_item)
{
	int height = c_radio->item_count;
	int width = 0;
	int buf = 0;
	int i;

	for (i=0;i<=c_radio->item_count-1;i++) {
		buf = strlen(c_radio->items[i].text);
		if (buf > width)
			width = buf;
	}
	width += 6;
	if ((box) && (width <= strlen(c_radio->title) + 2))
		width = strlen(c_radio->title) + 2;

	return(ngtk_show_radio(c_radio, height, width, top, left, cpair, attr,
						   box, def_item));
}

ngtk_radio_t *ngtk_auto_show_radio(ngtk_radio_t *c_radio, int cpair, int attr,
								   int box, int def_item)
{
	int top;
	int left;
	int height = c_radio->item_count;
	int width = 0;
	int buf = 0;
	int i;

	for (i=0;i<=c_radio->item_count-1;i++) {
		buf = strlen(c_radio->items[i].text);
		if (buf > width)
			width = buf;
	}
	width += 6;
	if ((box) && (width <= strlen(c_radio->title) + 2))
		width = strlen(c_radio->title) + 2;
	top = (LINES/2) - (height/2);
	left = (COLS/2) - (width/2);

	return(ngtk_show_radio(c_radio, height, width, top, left, cpair, attr,
						   box, def_item));
}
